
/******************************************************************************
* The MIT License
*
* Copyright (c) 2016, 2017 Paul Young
*
* Permission is hereby granted, free of charge, to any person
* obtaining a copy of this software and associated documentation
* files (the "Software"), to deal in the Software without
* restriction, including without limitation the rights to use, copy,
* modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
* EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
* NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
* BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
* ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
* CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*****************************************************************************/

// Code PracticeDlg.h : header file
//

#pragma once

#include "Keyer.h"
#include "Setup.h"
#include "Words.h"
#include "afxwin.h"


// CCodePracticeDlg dialog
class CCodePracticeDlg : public CDialogEx
{
// Construction
public:
	CCodePracticeDlg(CWnd* pParent = NULL);	// standard constructor

// Dialog Data
#ifdef AFX_DESIGN_TIME
	enum { IDD = IDD_CODEPRACTICE_DIALOG };
#endif

protected:
	virtual void DoDataExchange(CDataExchange* pDX);	// DDX/DDV support


// Implementation
protected:
	HICON m_hIcon;

	// Generated message map functions
	virtual BOOL OnInitDialog();
	afx_msg void OnSysCommand(UINT nID, LPARAM lParam);
	afx_msg void OnPaint();
	afx_msg HCURSOR OnQueryDragIcon();
	DECLARE_MESSAGE_MAP()

	CStatic m_speed_text;
	CStatic m_farnsworth_text;
	CStatic m_volume_text;
	CStatic m_frequency_text;

	CStatic m_speed_static;
	CStatic m_farnsworth_static;
	CStatic m_volume_static;
	CStatic m_frequency_static;

	CSliderCtrl m_speed_slider;
	CSliderCtrl m_farnsworth_slider;
	CSliderCtrl m_volume_slider;
	CSliderCtrl m_frequency_slider;

	CButton m_farnsworth_check;
	CButton m_success_check;
	CButton m_speed_check;

	CButton m_play_button;
	CButton m_setup_button;

	CStatic m_word_text;
	CStatic m_received_text;
	CStatic m_status_text;

	CFont m_word_font;
	CFont m_received_font;

	CWords m_words;
	CString m_word;
	CString m_received;

	const unsigned char *m_word_pointer;
	bool m_clear_received_text;
	int m_characters;

	bool m_speed_up;
	bool m_speed_down;
	bool m_word_scored;

	CSetup m_setup;
	
	Keyer_Type m_keyer_type;
	Keyer_Model m_keyer_model;

	CKeyer m_keyer;

	int m_port;

	ULONGLONG m_start;
	unsigned int m_correct;
	unsigned int m_incorrect;

	void set_speed(int delta);
	void new_word();
	void show_statistics();
	void reset_display_idle_timer() { SetThreadExecutionState(ES_DISPLAY_REQUIRED); }

	void enable_controls();
	void disable_controls();

public:
	afx_msg void OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar);
	afx_msg void OnBnClickedNewWord();
	afx_msg void OnBnClickedPlay();
	afx_msg void OnBnClickedSetup();
	afx_msg void OnBnClickedFarnsworth();
	afx_msg void OnBnClickedReset();

	LRESULT OnKeyerInternal(WPARAM wParam, LPARAM lParam);
	LRESULT OnKeyer(WPARAM wParam, LPARAM lParam);
};
